@@ -1,6 +1,6 @@
 
-/* $XConsortium: s3misc.c,v 1.1 94/03/28 21:16:11 dpw Exp $ */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/s3/s3misc.c,v 3.13 1994/09/23 10:09:59 dawes Exp $ */
+/* $XConsortium: s3misc.c,v 1.6 95/01/23 15:34:03 kaleb Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/s3/s3misc.c,v 3.22 1995/01/23 01:29:30 dawes Exp $ */
 /*
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  * 
@@ -57,10 +57,13 @@ static Bool s3TryAddress();
 extern ScreenPtr s3savepScreen;
 static PixmapPtr ppix = NULL;
 extern Bool  s3Localbus;
+extern Bool  s3VLB;
 extern Bool  s3LinearAperture;
 extern int s3BankSize;
 extern int s3DisplayWidth;
 extern pointer vgaBase;
+extern pointer vgaBaseLow;
+extern pointer vgaBaseHigh;
 extern pointer s3VideoMem;
 extern unsigned char s3Port59;
 extern unsigned char s3Port5A;
@@ -103,70 +106,22 @@ s3Initialize(scr_index, pScreen, argc, argv)
       s3BankSize = 0x10000;
       s3LinApOpt = 0x14;
 
-      /* First, map the vga window -- it is always required */
-      vgaBase = xf86MapVidMem(scr_index, VGA_REGION, (pointer)0xA0000,
-			      s3BankSize);
-      s3Init(s3InfoRec.modes);
-      /*
-       * The meaning of NOLINEAR_MODE has changed a little, and was
-       * used inconsistently here.  It now means don't attempt to map
-       * the framebuffer at high memory.  Some code here implied that at
-       * one stage it meant it was OK to use high memory, but only use a 64k
-       * aperture.
-       */
-#ifdef DEBUG
-      /* This is for debugging probe problem at > 8bpp (DHD) */
-      {
-	 long *poker;
-	 unsigned long pVal = 0x12345678;
-	 unsigned char reg53tmp;
-
-	 s3InitEnvironment();
-	 s3ImageWriteNoMem(0, 0, 4, 1, (char *) &pVal, 4, 0, 0,
-			   (short) s3alu[GXcopy], ~0);	 
-
-	 if (S3_801_928_SERIES (s3ChipId)) {
-	    int j;
-
-	    if (s3Mmio928) { /* Due to S3 bugs we must disable mmio */
-	       outb(vgaCRIndex, 0x53);
-	       reg53tmp = inb(vgaCRReg);
-	       outb(vgaCRReg, reg53tmp & ~0x10); /* save parallel bit */
-	    }
-	    /* begin 801 sequence for going in to linear mode */
-	    outb (vgaCRIndex, 0x40);
-	    /* enable fast write buffer and disable 8514/a mode */
-	    j = (s3Port40 & 0xf6) | 0x0a;
-	    outb (vgaCRReg, (unsigned char) j);
-	    outb(vgaCRIndex, 0x59);
-	    outb(vgaCRReg, 0x00);
-	    outb(vgaCRIndex, 0x5a);
-	    outb(vgaCRReg, 0x0a);
-	    s3LinApOpt = 0x14;
-	    outb (vgaCRReg, s3LinApOpt | s3SAM256);
-	    /* end  801 sequence to go into linear mode */
-	 }
-	 
-	 /* bank 0 should be selected */
-
-	 poker = (long *)vgaBase;
-	 s3TryAddress(poker, pVal, 0xa0000, 0);
-
-	 if (S3_801_928_SERIES (s3ChipId)) {
-	    /* begin 801  sequence to go into enhanced mode */
-	    outb (vgaCRIndex, 0x58);
-	    outb (vgaCRReg, s3SAM256);
-	    outb (vgaCRIndex, 0x40);
-	    outb (vgaCRReg, s3Port40);
-	    /* end 801 sequence to go into enhanced mode */
-	 }
-	 if (s3Mmio928) { /* Now re-enable mmio if required */
-	    outb(vgaCRIndex, 0x53);
-	    outb(vgaCRReg, reg53tmp | 0x10);
-	 }
+      if (OFLG_ISSET(OPTION_STB_PEGASUS, &s3InfoRec.options) &&
+	  !OFLG_ISSET(OPTION_NOLINEAR_MODE, &s3InfoRec.options) &&
+	  s3Mmio928) {
+	vgaBaseLow = xf86MapVidMem(scr_index, VGA_REGION,
+				   (pointer)0xA0000, s3BankSize);
+	vgaBaseHigh = xf86MapVidMem(scr_index, EXTENDED_REGION,
+				    (pointer)0x7C0A0000, s3BankSize);
+	vgaBase = vgaBaseLow;
+      } else {
+	/* First, map the vga window -- it is always required */
+	vgaBase = xf86MapVidMem(scr_index, VGA_REGION, (pointer)0xA0000,
+				s3BankSize);
       }
-#endif
-      
+
+      s3Init(s3InfoRec.modes);
+
       if (xf86LinearVidMem() &&
 	  !OFLG_ISSET(OPTION_NOLINEAR_MODE, &s3InfoRec.options)) {
 	 /* Now, see if we can map a high buffer */
@@ -181,26 +136,14 @@ s3Initialize(scr_index, pScreen, argc, argv)
 	    pVal = 0x12345678;
 
 	    s3InitEnvironment();
-	    s3ImageWriteNoMem(0, 0, 4, 1, (char *) &pVal, 4, 0, 0,
-			      (short) s3alu[GXcopy], ~0);	 
+	    s3ImageWriteNoMem(0, 0, 4 / s3Bpp, 1, (char *) &pVal, 4 / s3Bpp,
+			      0, 0, (short) s3alu[GXcopy], ~0);	 
 
 	    if (S3_801_928_SERIES (s3ChipId)) {
-	       int j;
-
-	       if (s3Mmio928) { /* Due to S3 bugs we must disable mmio */
-		  outb(vgaCRIndex, 0x53);
-		  reg53tmp = inb(vgaCRReg);
-		  outb(vgaCRReg, reg53tmp & ~0x10); /* save parallel bit */
-	       }
-	       /* begin 801 sequence for going in to linear mode */
-	       outb (vgaCRIndex, 0x40);
-	       /* enable fast write buffer and disable 8514/a mode */
-	       j = (s3Port40 & 0xf6) | 0x0a;
-	       outb (vgaCRReg, (unsigned char) j);
 	       outb(vgaCRIndex, 0x59);
 	       if (S3_x64_SERIES(s3ChipId)) 
 		  if (s3InfoRec.MemBase != 0) 
-		     outb(vgaCRReg, 0x03 | (s3InfoRec.MemBase>>24));
+		     outb(vgaCRReg, s3InfoRec.MemBase>>24);
 		  else
 		     outb(vgaCRReg, 0x03 | 0xf0);
 	       else
@@ -216,9 +159,7 @@ s3Initialize(scr_index, pScreen, argc, argv)
 		  s3LinApOpt=0x17;
 	       }
        	       s3BankSize = s3InfoRec.videoRam * 1024;
-	       /* go on to linear mode */
-	       outb (vgaCRReg, s3LinApOpt | s3SAM256);
-	       /* end  801 sequence to go into linear mode */
+	       s3EnableLinear();
 	    }
 	 
 	    /*
@@ -226,8 +167,8 @@ s3Initialize(scr_index, pScreen, argc, argv)
 	     * which values of LAW are decoded (see s3TryAddress() below).
 	     */
 	    if (OFLG_ISSET(OPTION_FB_DEBUG, &s3InfoRec.options)) {
-	       for (i = 0xfc; i >= 0; i-=4) {
-		  unsigned long addr = (i << 24) + (0x3<<24);
+	       for (i = 0xff; i >= 3; i--) { /* 4080Mb..48Mb stepsize 16Mb */
+		  unsigned long addr = (i << 24);
 
 	          s3VideoMem = xf86MapVidMem(scr_index, LINEAR_REGION,
 				          (pointer)addr, 4096);
@@ -239,44 +180,72 @@ s3Initialize(scr_index, pScreen, argc, argv)
 		  
 	    /*
 	     * If a MemBase value was given in the XF86Config, skip the LAW
-	     * probe and use the high 6 bits for the hw part of LAW.
+	     * probe and use the high 10 bits for the hw part of LAW.
+	     * Normally only 6 bits are set in hw, but the Diamond Stealth
+	     * Pro is different.
 	     */
 	    if (s3InfoRec.MemBase != 0) {
 	       unsigned long addr;
 
-	       /* Software part of LAW is 60MB = 0x3c00000 */
-	       addr = (s3InfoRec.MemBase & 0xfc000000) + 0x3c00000;
+	       addr = (s3InfoRec.MemBase & 0xffc00000);
 	       s3VideoMem = xf86MapVidMem(scr_index, LINEAR_REGION,
 					  (pointer)addr, s3BankSize);
+	       s3DisableLinear();
 	       outb(vgaCRIndex, 0x5a);
-	       outb(vgaCRReg, 0xC0);
-	       s3LinearAperture = TRUE;
-	       ErrorF("%s %s: Local bus LAW31-26 is %X\n", 
-		      XCONFIG_GIVEN, s3InfoRec.name, (addr >> 24) & 0xfc);
-	    } else {
 	       if (S3_x64_SERIES(s3ChipId)) {
+		  outb(vgaCRReg, (addr >> 16) & 0xff);
+		  outb(vgaCRIndex, 0x59);
+		  outb(vgaCRReg, (addr >> 24) & 0xff);
+	       }
+	       else 
+		  outb(vgaCRReg, 0xc0);
+	       s3EnableLinear();
+
+	       s3LinearAperture = TRUE;
+	       ErrorF("%s %s: Local bus LAW is 0x%03lXxxxxx\n", 
+		      XCONFIG_GIVEN, s3InfoRec.name, (addr >> 20));
+	    } else {
+	       if (S3_x64_SERIES(s3ChipId) && 
+		   !(S3_TRIO64_SERIES(s3ChipId) &&
+		     s3InfoRec.videoRam > 1024 && s3VLB)) {
 		  /* So far, only tested for the PCI ELSA W2000Pro */
 		  unsigned long addr;
 
+		  s3DisableLinear();
 	          outb(vgaCRIndex, 0x59);
 	          addr = inb(vgaCRReg) << 8;
 	          outb(vgaCRIndex, 0x5a);
 	          addr |= inb(vgaCRReg);
 	          addr <<= 16;
+		  s3EnableLinear();
+
                   if (OFLG_ISSET(OPTION_FB_DEBUG, &s3InfoRec.options)) {
 		     ErrorF("Read LAW as 0x%08X \n", addr);
 	          }
 		  s3VideoMem = xf86MapVidMem(scr_index, LINEAR_REGION,
                                              (pointer)addr, s3BankSize);
-		  s3LinearAperture = TRUE;
-                  ErrorF("%s %s: Local bus LAW is 0x%08lX\n", 
-                         XCONFIG_PROBED, s3InfoRec.name, addr);
+		  poker = (long *) s3VideoMem;
+		  if (s3TryAddress(poker, pVal, addr, 1)) {
+		     s3LinearAperture = TRUE;
+		     if (xf86Verbose) {
+			ErrorF("%s %s: Local bus LAW is 0x%08lX\n", 
+			       XCONFIG_PROBED, s3InfoRec.name, addr);
+		     }
+		  } else {
+		     s3LinearAperture = FALSE;
+		     CachedFrameBuffer = TRUE;
+		     if (xf86Verbose) {
+			ErrorF("%s %s: Local bus LAW is 0x%08lX %s\n", 
+			       XCONFIG_PROBED, s3InfoRec.name, addr,
+			       "but linear fb not usable");
+		     }
+		     xf86UnMapVidMem(scr_index, LINEAR_REGION, s3VideoMem,
+				     s3BankSize);
+		  }
 	       } else {
-	          for (i = 0xfc; i >= 0; i-=4) {
-		     unsigned long addr;
-	    
-		     /* Start at LAW(hw) + 48MB */
-		     addr = (i << 24) + (0x3<<24);
+	          for (i = 0xff; i >= 3; i--) { /* 4080Mb..48Mb stepsize 16Mb */
+		     unsigned long addr = (i << 24);
+
 		     s3VideoMem = xf86MapVidMem(scr_index, LINEAR_REGION,
 						(pointer)addr, s3BankSize);
 		     poker = (long *) s3VideoMem; 
@@ -284,9 +253,12 @@ s3Initialize(scr_index, pScreen, argc, argv)
 		     if (s3TryAddress(poker, pVal, addr, 1)) {
 			/* We found some ram, but is it ours? */
 	       
-			/* move it up by 12MB to LAW(hw) + 60MB */
+			s3DisableLinear();
+			/* move it up by 12MB */
 			outb(vgaCRIndex, 0x5a);
 			outb(vgaCRReg, 0xC0);
+			s3EnableLinear();
+
 			if (!s3TryAddress(poker, pVal, addr, 2)) {
 			   addr += (0x0C<<20);
 			   xf86UnMapVidMem(scr_index, LINEAR_REGION,
@@ -297,8 +269,29 @@ s3Initialize(scr_index, pScreen, argc, argv)
 		     
 			   if (s3TryAddress((long *)s3VideoMem, pVal,
 					    addr, 3)) {
-			      ErrorF("%s %s: Local bus LAW31-26 is %X\n", 
-				     XCONFIG_PROBED, s3InfoRec.name, i);
+			      ErrorF("%s %s: Local bus LAW is 0x%03Xxxxxx\n", 
+				     XCONFIG_PROBED, s3InfoRec.name,
+				     (i << 4 | 0x0c));
+			      s3LinearAperture = TRUE;
+			      break;
+			   }
+			   /*
+			    * Special test for Diamond Stealth Pro.
+			    * On this card, bits 6 and 7 of register 0x5a
+			    * control address bits A29 and A26 rather than
+			    * A22 and A23. Don't ask we why.
+			    */
+			   addr = (i << 24) | 0x24000000;
+			   xf86UnMapVidMem(scr_index, LINEAR_REGION,
+					   s3VideoMem, s3BankSize);
+			   s3VideoMem = xf86MapVidMem(scr_index, LINEAR_REGION,
+						      (pointer)addr,
+						      s3BankSize);
+			   if (s3TryAddress((long *)s3VideoMem, pVal,
+					    addr, 4)) {
+			      ErrorF("%s %s: Local bus LAW is 0x%03Xxxxxx\n", 
+				     XCONFIG_PROBED, s3InfoRec.name,
+				     (i | 0x24) << 4);
 			      s3LinearAperture = TRUE;
 			      break;
 			   }
@@ -309,6 +302,10 @@ s3Initialize(scr_index, pScreen, argc, argv)
 			   ErrorF("\t address range.\n");
 			   CachedFrameBuffer = TRUE;
 			}
+			s3DisableLinear();
+			outb(vgaCRIndex, 0x5a);
+			outb(vgaCRReg, 0x00);	/* reset for next probe */
+			s3EnableLinear();
 		     }
 		     xf86UnMapVidMem(scr_index, LINEAR_REGION, s3VideoMem,
 				     s3BankSize);
@@ -328,18 +325,7 @@ s3Initialize(scr_index, pScreen, argc, argv)
 	       s3BankSize = 0x10000;
 	       s3VideoMem = NULL;
 	    }
-	    if (S3_801_928_SERIES (s3ChipId)) {
-	       /* begin 801  sequence to go into enhanced mode */
-	       outb (vgaCRIndex, 0x58);
-	       outb (vgaCRReg, s3SAM256);
-	       outb (vgaCRIndex, 0x40);
-	       outb (vgaCRReg, s3Port40);
-	       /* end 801 sequence to go into enhanced mode */
-	    }
-	    if (s3Mmio928) { /* Now re-enable mmio if required */
-	       outb(vgaCRIndex, 0x53);
-	       outb(vgaCRReg, reg53tmp | 0x10);
-	    }
+            s3DisableLinear();
          }
       }
 
@@ -608,22 +594,45 @@ s3SwitchMode(mode)
 void
 s3AdjustFrame(int x, int y)
 {
-   int   Base;
+   int   Base, origBase;
    unsigned char tmp;
+   extern int s3AdjustCursorXPos;  /* for s3Cursor.c */
+   extern int s3HDisplay;
 
    if (OFLG_ISSET(OPTION_SHOWCACHE, &s3InfoRec.options)) {
       if ( debugcache & 1)
          y += 512;
    }
       
-   Base = ((y * s3DisplayWidth + x) * s3Bpp) >> 2;
+   if (x > s3DisplayWidth - s3HDisplay)
+      x  = s3DisplayWidth - s3HDisplay;
 
-   /* Elsa Winner 1000 has display errors with 16bpp and (Base&0x0f) == 0x0f */
-   if (s3Bpp>1 && DAC_IS_SC15025 && S3_928_SERIES(s3ChipId)) {
-      if ((Base&0x0f) == 0x0f) Base--;
+   /* so may S3 cards have problems with some odd base addresses, 
+    * to catch them all only even base values will be used.
+    */
+
+   origBase = (y * s3DisplayWidth + x) * s3Bpp;
+   Base = (origBase >> 2) & ~1;
+
+   if (S3_964_SERIES(s3ChipId) && DAC_IS_BT485_SERIES) {
+      if ((Base & 0x3f) >= 0x3c) 
+	 Base = (Base & ~0x3f) | 0x3b;
+      else if (s3Bpp>1 && (Base & 0x3f) == 0x3a) 
+	 Base = (Base & ~0x3f) | 0x39;
+      else if (s3Bpp>2 && (Base & 0x1f) == 0x1a) 
+	 Base = (Base & ~0x1f) | 0x19;
    }
-   else if (DAC_IS_SC15025 && S3_928_SERIES(s3ChipId)) {
-      if ((Base&0x3f) == 0x3f) Base--;
+
+   if (S3_964_SERIES(s3ChipId) && DAC_IS_TI3025) {
+      int px, py, a;
+      miPointerPosition(&px, &py);
+      if (s3Bpp==1)
+	 a = 4-1;
+      else 
+	 a = 8-1;
+      if (px-x > s3HDisplay/2)
+	 Base = ((origBase + a*4) >> 2) & ~1;
+      Base &= ~a;
    }
 
    outb(vgaCRIndex, 0x31);
@@ -638,6 +647,8 @@ s3AdjustFrame(int x, int y)
    outw(vgaCRIndex, (Base & 0x00FF00) | 0x0C);
    outw(vgaCRIndex, ((Base & 0x00FF) << 8) | 0x0D);
 
+   s3AdjustCursorXPos = (origBase - (Base << 2)) / s3Bpp;
+
    if (s3ModeSwitched) {
       s3ModeSwitched = FALSE;
       s3RestoreCursor(s3savepScreen);
