@@ -1,5 +1,5 @@
-/* $XConsortium: xf86Events.c,v 1.2 94/03/31 11:11:56 dpw Exp $ */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Events.c,v 3.0 1994/05/08 05:20:49 dawes Exp $ */
+/* $XConsortium: xf86Events.c,v 1.11 95/01/16 13:16:59 kaleb Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Events.c,v 3.8 1995/01/11 03:50:36 dawes Exp $ */
 /*
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  *
@@ -313,7 +313,7 @@ xf86PostKbdEvent(key)
   KeySym      *keysym;
   int         keycode;
   static int  lockkeys = 0;
-#if defined(SYSCONS_SUPPORT)
+#if defined(SYSCONS_SUPPORT) || defined(PCVT_SUPPORT)
   static Bool first_time = TRUE;
 #endif
 
@@ -322,11 +322,12 @@ xf86PostKbdEvent(key)
     scanCode = xf86CodrvMap[scanCode];
 #endif
 
-#if defined(SYSCONS_SUPPORT)
+#if defined(SYSCONS_SUPPORT) || defined(PCVT_SUPPORT)
   if (first_time)
   {
     first_time = FALSE;
-    VTSwitchEnabled = (xf86Info.consType == SYSCONS);
+    VTSwitchEnabled = (xf86Info.consType == SYSCONS)
+	    || (xf86Info.consType == PCVT);
   }
 #endif
   /*
@@ -338,12 +339,13 @@ xf86PostKbdEvent(key)
       
     case KEY_Prefix0:
     case KEY_Prefix1:
-#if defined(PCCONS_SUPPORT) || defined(SYSCONS_SUPPORT)
-      if (xf86Info.consType == PCCONS || xf86Info.consType == SYSCONS) {
+#if defined(PCCONS_SUPPORT) || defined(SYSCONS_SUPPORT) || defined(PCVT_SUPPORT)
+      if (xf86Info.consType == PCCONS || xf86Info.consType == SYSCONS
+	  || xf86Info.consType == PCVT) {
 #endif
         xf86Info.scanPrefix = scanCode;  /* special prefixes */
         return;
-#if defined(PCCONS_SUPPORT) || defined(SYSCONS_SUPPORT)
+#if defined(PCCONS_SUPPORT) || defined(SYSCONS_SUPPORT) || defined(PCVT_SUPPORT)
       }
       break;
 #endif
@@ -353,32 +355,48 @@ xf86PostKbdEvent(key)
       updateLeds = TRUE;              /* led changes by firmware */
       break;
     }
-    if (xf86Info.serverNumLock &&
-        ((!xf86Info.numLock && !ModifierDown(ShiftMask)) ||
-         (xf86Info.numLock && ModifierDown(ShiftMask)))) {
+    if (xf86Info.serverNumLock) {
+     if ((!xf86Info.numLock && ModifierDown(ShiftMask)) ||
+         (xf86Info.numLock && !ModifierDown(ShiftMask))) {
       /*
        * Hardwired numlock handling ... (Some applications break if they have
        * these keys double defined, like twm)
        */
       switch (scanCode) {
-      case KEY_KP_7:        scanCode = KEY_Home;      break;  /* curs home */
-      case KEY_KP_8:        scanCode = KEY_Up;        break;  /* curs up */
-      case KEY_KP_9:        scanCode = KEY_PgUp;      break;  /* curs pgup */
-      case KEY_KP_4:        scanCode = KEY_Left;      break;  /* curs left */
-      case KEY_KP_5:        scanCode = KEY_Begin;     break;  /* curs begin */
-      case KEY_KP_6:        scanCode = KEY_Right;     break;  /* curs right */
-      case KEY_KP_1:        scanCode = KEY_End;       break;  /* curs end */
-      case KEY_KP_2:        scanCode = KEY_Down;      break;  /* curs down */
-      case KEY_KP_3:        scanCode = KEY_PgDown;    break;  /* curs pgdown */
-      case KEY_KP_0:        scanCode = KEY_Insert;    break;  /* curs insert */
-      case KEY_KP_Decimal:  scanCode = KEY_Delete;    break;  /* curs delete */
+      case KEY_KP_7:        scanCode = KEY_SN_KP_7;   break;  /* curs 7 */
+      case KEY_KP_8:        scanCode = KEY_SN_KP_8;   break;  /* curs 8 */
+      case KEY_KP_9:        scanCode = KEY_SN_KP_9;   break;  /* curs 9 */
+      case KEY_KP_4:        scanCode = KEY_SN_KP_4;   break;  /* curs 4 */
+      case KEY_KP_5:        scanCode = KEY_SN_KP_5;   break;  /* curs 5 */
+      case KEY_KP_6:        scanCode = KEY_SN_KP_6;   break;  /* curs 6 */
+      case KEY_KP_1:        scanCode = KEY_SN_KP_1;   break;  /* curs 1 */
+      case KEY_KP_2:        scanCode = KEY_SN_KP_2;   break;  /* curs 2 */
+      case KEY_KP_3:        scanCode = KEY_SN_KP_3;   break;  /* curs 3 */
+      case KEY_KP_0:        scanCode = KEY_SN_KP_0;   break;  /* curs 0 */
+      case KEY_KP_Decimal:  scanCode = KEY_SN_KP_Dec; break;  /* curs decimal */
       }
+     } else {
+      switch (scanCode) {
+      case KEY_KP_7:        scanCode = KEY_SN_KP_Home;  break;  /* curs home */
+      case KEY_KP_8:        scanCode = KEY_SN_KP_Up  ;  break;  /* curs up */
+      case KEY_KP_9:        scanCode = KEY_SN_KP_Prior; break;  /* curs pgup */
+      case KEY_KP_4:        scanCode = KEY_SN_KP_Left;  break;  /* curs left */
+      case KEY_KP_5:        scanCode = KEY_SN_KP_Begin; break;  /* curs begin */
+      case KEY_KP_6:        scanCode = KEY_SN_KP_Right; break;  /* curs right */
+      case KEY_KP_1:        scanCode = KEY_SN_KP_End;   break;  /* curs end */
+      case KEY_KP_2:        scanCode = KEY_SN_KP_Down;  break;  /* curs down */
+      case KEY_KP_3:        scanCode = KEY_SN_KP_Next;  break;  /* curs pgdn */
+      case KEY_KP_0:        scanCode = KEY_SN_KP_Ins;   break;  /* curs ins */
+      case KEY_KP_Decimal:  scanCode = KEY_SN_KP_Del;   break;  /* curs del */
+      }
+     }
     }
   }
 
   else if (
-#if defined(__386BSD__) || defined(__NetBSD__) || defined(__FreeBSD__)
-           (xf86Info.consType == PCCONS || xf86Info.consType == SYSCONS) &&
+#ifdef CSRG_BASED
+           (xf86Info.consType == PCCONS || xf86Info.consType == SYSCONS
+	    || xf86Info.consType == PCVT) &&
 #endif
            (xf86Info.scanPrefix == KEY_Prefix0)) {
     xf86Info.scanPrefix = 0;
@@ -401,6 +419,9 @@ xf86PostKbdEvent(key)
     case KEY_Slash:       scanCode = KEY_KP_Divide; break;  /* keyp divide */
     case KEY_Alt:         scanCode = KEY_AltLang;   break;  /* right alt */
     case KEY_ScrollLock:  scanCode = KEY_Break;     break;  /* curs break */
+    case 0x5b:            scanCode = KEY_LMeta;     break;
+    case 0x5c:            scanCode = KEY_RMeta;     break;
+    case 0x5d:            scanCode = KEY_Menu;      break;
       /*
        * Ignore virtual shifts (E0 2A, E0 AA, E0 36, E0 B6)
        */
@@ -432,7 +453,7 @@ xf86PostKbdEvent(key)
 	
       case KEY_BackSpace:
 	if (!xf86Info.dontZap) GiveUp(0);
-	return;
+	break;
 	
 	/*
 	 * The idea here is to pass the scancode down to a list of
@@ -440,14 +461,20 @@ xf86PostKbdEvent(key)
 	 * for processing certain keys.
 	 */
       case KEY_KP_Minus:   /* Keypad - */
-	if (down) xf86ZoomViewport(xf86Info.currentScreen, -1);
-	return;
+	if (!xf86Info.dontZoom) {
+	  if (down) xf86ZoomViewport(xf86Info.currentScreen, -1);
+	  return;
+	}
+	break;
 	
       case KEY_KP_Plus:   /* Keypad + */
-	if (down) xf86ZoomViewport(xf86Info.currentScreen,  1);
-	return;
+	if (!xf86Info.dontZoom) {
+	  if (down) xf86ZoomViewport(xf86Info.currentScreen,  1);
+	  return;
+	}
+	break;
 
-#if defined(linux) || defined(__386BSD__) || defined(__NetBSD__) || defined(__FreeBSD__)
+#if defined(linux) || (defined(CSRG_BASED) && (defined(SYSCONS_SUPPORT) || defined(PCVT_SUPPORT)))
 	/*
 	 * Under Linux, the raw keycodes are consumed before the kernel
 	 * does any processing on them, so we must emulate the vt switching
@@ -463,7 +490,11 @@ xf86PostKbdEvent(key)
       case KEY_F8:
       case KEY_F9:
       case KEY_F10:
-        if (VTSwitchEnabled && !xf86Info.vtSysreq)
+        if (VTSwitchEnabled && !xf86Info.vtSysreq
+#if (defined(CSRG_BASED) && (defined(SYSCONS_SUPPORT) || defined(PCVT_SUPPORT)))
+	    && (xf86Info.consType == SYSCONS || xf86Info.consType == PCVT)
+#endif
+	    )
         {
 	  if (down)
             ioctl(xf86Info.consoleFd, VT_ACTIVATE, scanCode - KEY_F1 + 1);
@@ -472,25 +503,29 @@ xf86PostKbdEvent(key)
 	break;
       case KEY_F11:
       case KEY_F12:
-        if (VTSwitchEnabled && !xf86Info.vtSysreq)
+        if (VTSwitchEnabled && !xf86Info.vtSysreq
+#if (defined(CSRG_BASED) && (defined(SYSCONS_SUPPORT) || defined(PCVT_SUPPORT)))
+	    && (xf86Info.consType == SYSCONS || xf86Info.consType == PCVT)
+#endif
+	    )
         {
 	  if (down)
             ioctl(xf86Info.consoleFd, VT_ACTIVATE, scanCode - KEY_F11 + 11);
           return;
         }
 	break;
-#endif
+#endif /* linux || BSD with VTs */
 
       /* just worth mentioning here: any 386bsd keyboard driver
        * (pccons.c or co_kbd.c) catches CTRL-ALT-DEL and CTRL-ALT-ESC
        * before any application (e.g. XF86) will see it
-       * OBS: syscons does not !
+       * OBS: syscons does not, nor does pcvt !
        */
       } 
     }
 
     /*
-     * Start of acutal Solaris VT switching code.  
+     * Start of actual Solaris VT switching code.  
      * This should pretty much emulate standard SVR4 switching keys.
      * 
      * DWH 12/2/93
@@ -721,7 +756,7 @@ xf86PostKbdEvent(key)
    * normal, non-keypad keys
    */
   if (scanCode < KEY_KP_7 || scanCode > KEY_KP_Decimal) {
-#if !defined(__BSD__) && !defined(MACH386) && !defined(MINIX) && !defined(__OSF__)
+#if !defined(CSRG_BASED) && !defined(MACH386) && !defined(MINIX) && !defined(__OSF__)
     /*
      * magic ALT_L key on AT84 keyboards for multilingual support
      */
@@ -732,7 +767,7 @@ xf86PostKbdEvent(key)
 	UsePrefix = TRUE;
 	Direction = TRUE;
       }
-#endif /* !MACH386 && !__BSD__ && !MINIX && !__OSF__ */
+#endif /* !CSRG_BASED && !MACH386 && !MINIX && !__OSF__ */
   }
 
 
@@ -945,8 +980,7 @@ xf86SigHandler(signo)
 {
   signal(signo,SIG_IGN);
   xf86Info.caughtSignal = TRUE;
-  ErrorF("Caught signal %d.  Server aborting\n", signo);
-  AbortDDX();
+  FatalError("Caught signal %d.  Server aborting\n", signo);
 }
 
 /*
