@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/vga256/drivers/cirrus/cir_alloc.h,v 3.1 1994/08/20 08:48:30 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/vga256/drivers/cirrus/cir_alloc.h,v 3.2 1995/01/04 04:42:15 dawes Exp $ */
 
 /*
  * Definitions for video memory allocator in cir_alloc.c.
@@ -10,16 +10,18 @@ _XFUNCPROTOBEGIN
 
 #if NeedFunctionPrototypes
 
-void CirrusInitializeAllocator(int base);
+int CirrusInitializeAllocator(int base);
 int CirrusAllocate(int size);
 void CirrusFree(int vidaddr);
 void CirrusUploadPattern(unsigned char *pattern, int w, int h, int vidaddr,
 	int srcpitch);
+int CirrusCursorAllocate ( cirrusCurRecPtr cirrusCur );
 
 #else
 
-void CirrusInitializeAllocator();
-void CirrusAllocate();
+int CirrusInitializeAllocator();
+int CirrusAllocate();
+int CirrusCursorAllocate ();
 void CirrusFree();
 void CirrusUploadPattern();
 
