@@ -1,5 +1,5 @@
-/* $XConsortium: vga.c,v 1.1 94/03/28 21:55:24 dpw Exp $ */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/vga256/vga/vga.c,v 3.22 1994/09/24 15:15:18 dawes Exp $ */
+/* $XConsortium: vga.c,v 1.6 95/01/16 13:18:27 kaleb Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/vga256/vga/vga.c,v 3.28 1995/01/11 03:52:41 dawes Exp $ */
 /*
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  *
@@ -79,6 +79,7 @@ ScrnInfoRec vga256InfoRec = {
   -1,			/* int scrnIndex */
   vgaProbe,		/* Bool (* Probe)() */
   vgaScreenInit,	/* Bool (* Init)() */
+  vgaValidMode,		/* Bool (* ValidMode)() */
   vgaEnterLeaveVT,	/* void (* EnterLeaveVT)() */
   (void (*)())NoopDDA,		/* void (* EnterLeaveMonitor)() */
   (void (*)())NoopDDA,		/* void (* EnterLeaveCursor)() */
@@ -150,6 +151,9 @@ ScrnInfoRec vga256InfoRec = {
   0,			/* int COPbase */
   0,			/* int POSbase */
   0,			/* int instance */
+  0,			/* int s3Madjust */
+  0,			/* int s3Nadjust */
+  0,			/* int s3MClk */
 };
 
 pointer vgaOrigVideoState = NULL;
@@ -168,6 +172,11 @@ Bool (* vgaInitFunc)(
     DisplayModePtr
 #endif
 ) = (Bool (*)())NoopDDA;
+Bool (* vgaValidModeFunc)(
+#if NeedFunctionPrototypes
+    DisplayModePtr
+#endif
+) = (Bool (*)())NoopDDA;
 void * (* vgaSaveFunc)(
 #if NeedFunctionPrototypes
     void *
@@ -230,6 +239,13 @@ static void (* saveSetReadWriteFunc)();
 
 vgaHWCursorRec vgaHWCursor;
 
+#ifdef MONOVGA
+static int validDepth = 1;
+#endif
+#ifdef XF86VGA16
+static int validDepth = 4;
+#endif
+
 extern miPointerScreenFuncRec xf86PointerScreenFuncs;
 extern int defaultColorVisualClass;
 
@@ -373,6 +389,13 @@ vgaProbe()
 	  return(FALSE);
       }
   }
+#else
+  if (vga256InfoRec.depth != validDepth) {
+    ErrorF("\n%s %s: Unsupported bpp for %s server (%d)\n",
+           XCONFIG_GIVEN, vga256InfoRec.name, vga256InfoRec.name,
+           vga256InfoRec.depth);
+	  return(FALSE);
+  }
 #endif
 
   for (i=0; Drivers[i]; i++)
@@ -489,6 +512,7 @@ vgaProbe()
 
 	vgaEnterLeaveFunc = Drivers[i]->ChipEnterLeave;
 	vgaInitFunc = Drivers[i]->ChipInit;
+	vgaValidModeFunc = Drivers[i]->ChipValidMode;
 	vgaSaveFunc = Drivers[i]->ChipSave;
 	vgaRestoreFunc = Drivers[i]->ChipRestore;
 	vgaAdjustFunc = Drivers[i]->ChipAdjust;
@@ -1158,7 +1182,7 @@ vgaEnterLeaveVT(enter, screen_idx)
   else
     {
 
-      /* Make sure that another dirver hasn't disabeled IO */    
+      /* Make sure that another driver hasn't disabeled IO */    
       xf86MapDisplay(screen_idx, VGA_REGION);
       if (vgaUseLinearAddressing)
         xf86MapDisplay(screen_idx, LINEAR_REGION);
@@ -1224,8 +1248,8 @@ vgaEnterLeaveVT(enter, screen_idx)
       
       vgaInitFunc = (Bool (*)())saveDummy;
       vgaSaveFunc = (void * (*)())saveDummy;
-      vgaRestoreFunc = saveDummy;
-      vgaAdjustFunc = saveDummy;
+      vgaRestoreFunc = (void (*)())saveDummy;
+      vgaAdjustFunc = (void (*)())saveDummy;
       vgaSaveScreenFunc = saveDummy;
       vgaSetReadFunc = saveDummy;
       vgaSetWriteFunc = saveDummy;
@@ -1283,6 +1307,10 @@ vgaSwitchMode(mode)
   if ((vgaInitFunc)(mode))
   {
     vgaRestore(vgaNewVideoState);
+    if (vgaHWCursor.Initialized)
+    {
+      vgaHWCursor.Restore(savepScreen);
+    }
     return(TRUE);
   }
   else
@@ -1291,3 +1319,22 @@ vgaSwitchMode(mode)
     return(FALSE);
   }
 }
+
+/*
+ * vgaValidMode --
+ *     Validate a mode for VGA architecture. Also checks the chip driver
+ *     to see if the mode can be supported.
+ */
+Bool
+vgaValidMode(mode)
+     DisplayModePtr mode;
+{
+  if ((vgaValidModeFunc)(mode))
+  {
+    return(TRUE);
+  }
+  else
+  {
+    return(FALSE);
+  }
+}
